#!/usr/bin/env python3
import argparse
import sys

def args_parser(args):
    parser = argparse.ArgumentParser(        description='Calculate digisys HW1 variant')
    parser.add_argument("sid", help="6-digit student ID (e.g., 123456)", type=str)
    parser.add_argument('--print_tt', default=False,
        action='store_true', help="Print function-system truthtable")
    parser.add_argument('--to_file', default=False,
        action='store_true', help="Generate espresso input file")
    # ...Create your parser as you like...
    return parser.parse_args(args)

def generate_fn_response(response: dict, input_bits: int = 4):
    fn_list = []
    for i in range(2 ** input_bits):
        x = hex(i)[2:]
        if x in response.get('ones'):
            fn_list.append(str(1))
        elif x in response.get('dontcare'):
            fn_list.append('-')
        else:
            fn_list.append(str(0))
    return fn_list

def calc_function_hex(sid: int, mult: int = 5, target_len: int = 8):
    x = sid
    target_num_len = target_len + 2
    while True:
        x = x * mult
        if hex(x).__len__() == target_num_len:
            break
    ones = set(list(hex(x)[2:]))
    dontc = set(list(hex(x//3)[2:]))
    dontc = dontc-ones
    return {'ones': sorted(ones), 'dontcare': sorted(dontc)}

def calc_hw1_fns(sid: int, mults: list, target_len: int = 8):
    fns = []
    for x in mults:
        fns.append(calc_function_hex(sid, x, target_len))
    return fns

def print_fns(fns: list):
    for idx, fn in enumerate(fns):
        print(f"fn{idx+1}> 1: {','.join(fn.get('ones'))} | -: {','.join(fn.get('dontcare'))}")

def generate_tt(fns: list, input_bits: int = 4):
    temp = [generate_fn_response(x, input_bits) for x in fns]
    tt = [''.join(x) for x in zip(*temp)]
    return tt

def print_tt(tt):
    print(f"{''.join((str(idx+1) for idx in range(len(tt[0]))))}")
    for x in tt:
        print(x)

def generate_espresso_file(sid: int, tt: list, input_bits: int = 4):
    tt_input = []
    for idx, x in enumerate(tt):
        tt_input.append(f"{format(idx,'b').zfill(input_bits)} {x}")
    file_name = f"hw1_{sid:06}.txt"
    with open(file_name, 'w') as f:
        f.write(f".i {input_bits}\n")
        f.write(f".o {input_bits}\n")
        f.write(f".ilb {' '.join([f'x{i+1}' for i in range(input_bits)])}\n")
        f.write(f".ob {' '.join([f'y{i+1}' for i in range(input_bits)])}\n")
        for x in tt_input:
            f.write(f"{x}\n")
        f.write(".e\n")
    print(f"Generated {file_name}")


def main(sid: int, show_tt: bool = False, to_file: bool = False,
         input_bits: int = 4, mults=None, target_len: int = 8):
    if mults is None:
        mults = [5, 7, 11, 13]
    print(f"{str(sid).zfill(6)}")
    fns = calc_hw1_fns(sid, mults, target_len)
    print_fns(fns)
    tt = generate_tt(fns, input_bits)
    if show_tt:
        print("\nFunction-system truthtable")
        print_tt(tt)
        print("\n")
    if to_file:
        generate_espresso_file(sid, tt, input_bits)


if __name__ == '__main__':
    args = args_parser(sys.argv[1:])
    to_file = False
    show_tt = False
    if not args.sid.isnumeric():
        print("Student ID must be numeric")
        print("Example: 123456")
        sys.exit(1)
    if args.sid.__len__() != 6:
        print("Invalid student ID length")
        print("Please use 6-digit student ID")
        sys.exit(2)
    if args.to_file:
        to_file = True
    if args.print_tt:
        show_tt = True
    main(int(args.sid), show_tt, to_file)
